-- From Natively Compiled T-SQL: The Fastest T-SQL Ever 
-- (C) Copyright  2016 Andrew S Novick.  All rights reserved
-- Use for demo purposes only.

drop table EAV_inmem 
GO

CREATE TABLE EAV_inmem  (
	[entity_id] int NOT NULL,
	attribute_id smallint NOT NULL ,
	value VARCHAR(256) NOT NULL,
	begin_datetime datetime NOT NULL DEFAULT (GETDATE()),
	end_datetime datetime NULL,
	CONSTRAINT PK_entity_attribute PRIMARY KEY NONCLUSTERED (attribute_id, begin_datetime, [entity_id])
   , INDEX ix_end_dateime NONCLUSTERED ([entity_id], [attribute_id], end_datetime)
    ) WITH (MEMORY_OPTIMIZED = ON, 	DURABILITY=SCHEMA_AND_DATA);
go

CREATE TYPE EAV_upsert_begin_datetime  AS TABLE (
	begin_datetime datetime NOT NULL 
	, index ix_begin_datetime NONCLUSTERED (begin_datetime)
) WITH (MEMORY_OPTIMIZED = ON)
go

CREATE PROC dbo.EAV_Upsert (@entity_id int NOT NULL
   , @attribute_id smallint NOT NULL 
   , @value varchar(255) NOT NULL 
   , @update_datetime datetime OUTPUT
   , @previous_begin_datetime datetime = NULL OUTPUT 
   , @previous_value varchar(256) = NULL OUTPUT
   )
     WITH NATIVE_COMPILATION, SCHEMABINDING
AS BEGIN ATOMIC WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT,  
                               language = N'English')

DECLARE @upsert_begin_datetime EAV_upsert_begin_datetime; 

SELECT @previous_begin_datetime = begin_datetime 
     , @previous_value = [value]
	FROM dbo.EAV_inmem 
	WHERE [entity_id] = @entity_id
	  AND [attribute_id] = @attribute_id 
	  and end_datetime IS NULL

if @previous_value IS NOT NULL 
	and @pervious_value != @value BEGIN -- found a row
	
	UPDATE dbo.EAV_inmem 
		SET end_datetime = GETDATE()
		OUTPUT inserted.end_datetime INTO @upsert_begin_datetime (begin_datetime)
		WHERE [entity_id] = @entity_id
		  AND [attribute_id] = @attribute_id 
		  and end_datetime IS NULL 
		  AND [value] != @value

	SELECT TOP 1 @update_datetime = begin_datetime FROM @upsert_begin_datetime

	END
ELSE BEGIN 
	set @update_datetime = GETDATE();
END

INSERT dbo.EAV_nmem ([entity_id], attribute_id, [value], begin_datetime)
		   values (@entity_id, @attribute_id, @value, @update_datetime)

END

go

drop type EAV_tt_inmem
go
CREATE TYPE EAV_tt_inmem  AS TABLE (
	[entity_id] int NOT NULL,
	attribute_id smallint NOT NULL ,
	value VARCHAR(256) NOT NULL,
    INDEX IX_entity_attribute NONCLUSTERED ([entity_id], attribute_id)
) WITH (MEMORY_OPTIMIZED = ON)
go

DECLARE @eav_temp EAV_tt_inmem;

drop table eav_temp_inmem
go
CREATE TABLE EAV_temp_inmem  (
	[entity_id] int NOT NULL,
	attribute_id smallint NOT NULL ,
	value VARCHAR(256) NOT NULL,
	session_id smallint not null DEFAULT (@@SPID),
	INDEX IL_session_id (session_id) ,
	INDEX IX_entity_attribute
            NONCLUSTERED ([entity_id], attribute_id),
    CONSTRAINT CHK_temp1_session_id 
              CHECK ( session_id = @@spid )
    ) WITH (MEMORY_OPTIMIZED = ON, 			                    DURABILITY=SCHEMA_ONLY);
go


go
CREATE FUNCTION dbo.fn_SessionFilter(@session_id smallint)
    RETURNS TABLE
WITH SCHEMABINDING, NATIVE_COMPILATION
AS
    RETURN SELECT 1 as fn_SessionFilter WHERE @session_id=@@spid;
GO
CREATE SECURITY POLICY dbo.temp1Filter
	ADD FILTER PREDICATE dbo.fn_SessionFilter(session_id)
    ON dbo.EAV_temp_inmem
    WITH (STATE = ON);
GO

CREATE PROC dbo.myProc (@myStr VARCHAR(64) NOT NUL)
                                         WITH NATIVE_COMPILATION, SCHEMABINDING
AS BEGIN ATOMIC WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT                                                language = N'English')

		-- BODY GOES HERE
END
go

CREATE PROC dbo.myProc (@myStr VARCHAR(64) NOT NULL)
                             WITH NATIVE_COMPILATION, SCHEMABINDING
AS BEGIN ATOMIC WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT, language = N'English')

	DECLARE @eav_temp dbo.EAV_tt_inmem;

	insert dbo.EAV_temp_inmem (entity_id, attribute_id, value)
		OUTPUT inserted.entity_id, inserted.attribute_id, inserted.[value]
			into @eav_temp ([entity_id], attribute_id, [value])
--insert EAV_inmem (entity_id, attribute_id, value)
values (1,2,'abcd'), (2,3,'defg')		
--values (1,2,'abcd')
			-- BODY GOES HERE

	SELECT [entity_id], attribute_id, [value] FROM @eav_temp

END
go

-----------------------------------------------------------------------------------
-- This is the error filled version that I first typed

CREATE PROC dbo.myProc (@myStr VARCHAR(64) NOT NULL)
                             WITH NATIVE_COMPILATION, SCHEMABINDING
AS BEGIN ATOMIC WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT/*, language = N'English'*/)

	DECLARE @eav_temp EAV_tt_inmem;

	insert EAV_temp_inmem (entity_id, attribute_id, value)
		OUTPUT inserted.entity_id, inserted.attribute_id, inserted.[value]
			into @eav_temp ([entity_id], attribute_id, [value])
		values (1,2,'abcd')

	SELECT * FROM @eav_temp
	--SELECT [entity_id], attribute_id, [value] FROM @eav_temp

END

go 

select db_id() [db_id], OBJECT_ID('dbo.myProc') [myProc], OBJECT_ID('dbo.EAV_temp_inmem') [EAV_temp_inmem]

