use mkt 
-- From Natively Compiled T-SQL: The Fastest T-SQL Ever 
-- (C) Copyright  2016 Andrew S Novick.  All rights reserved
-- Use for demo purposes only.

-- The data comes from Quandl's Wiki Market End-of-Day dataset, which is available for free.  But you could use almost any data 


-- Add a Filegroup to hold memory optimized tables
alter database mkt add filegroup mkt_inmem_fg CONTAINS MEMORY_OPTIMIZED_DATA
go
-- and a file to hold them.
alter database mkt add file (name=mkt_inmem_1, FILENAME = 'C:\mkt_db\In_mem_1') to filegroup mkt_inmem_fg
go

SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
SET ANSI_PADDING ON
GO


--- This is th disk based table
CREATE TABLE [dbo].[wiki_mkt_close_disk](
	[ticker] [varchar](100) NOT NULL,
	[asof_date] [date] NOT NULL,
	[adj_close] [float] NULL,
	[adj_close_ema] [float] NULL,
 CONSTRAINT [pk_wiki_mkt_close_disk] PRIMARY KEY CLUSTERED 
(
	[ticker] ASC,  	[asof_date] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]
go

CREATE TABLE [dbo].[wiki_mkt_close_inmem](
	[ticker] [varchar](100) NOT NULL,
	[asof_date] [date] NOT NULL,
	[adj_close] [float] NULL,
	[adj_close_ema] [float] NULL,
 CONSTRAINT [pk_wiki_mkt_close_inmem] PRIMARY KEY NONCLUSTERED (ticker, asof_date)
 , INDEX ix_asof_date_ticker NONCLUSTERED (asof_date, ticker)
 ) WITH (MEMORY_OPTIMIZED = ON, 	DURABILITY=SCHEMA_AND_DATA)


select count(*) from wiki_mkt_close_disk -- <<<< this is the source data table

truncate table wiki_mkt_close_inmem  -- doesn't work with inmem
delete from wiki_mkt_close_inmem

 --- populate first day
 insert into wiki_mkt_close_inmem (ticker, asof_date, adj_close, adj_close_ema)
	select ticker, asof_date, adj_close, adj_close_ema
	from wiki_mkt_close_disk where asof_date = '2016-06-01'
go

update wiki_mkt_close_inmem set adj_close_ema = dbo.exp_moving_avg_scalar_nc(null, adj_close, 21, 2)
select * from wiki_mkt_close_inmem
go

drop table if exists [dbo].[wiki_mkt_close_staging_inmem]

go

CREATE TABLE [dbo].[wiki_mkt_close_staging_inmem](
	[ticker] [varchar](100) NOT NULL,
	[adj_close] [float] NULL,
  CONSTRAINT [pk_wiki_mkt_close_staging_inmem] PRIMARY KEY NONCLUSTERED (ticker)
 ) WITH (MEMORY_OPTIMIZED = ON, 	DURABILITY=SCHEMA_AND_DATA)
go

delete from wiki_mkt_close_staging_inmem 
insert into wiki_mkt_close_staging_inmem 
	select ticker, adj_close 
	from wiki_mkt_close_disk 
	where asof_date = '2016-06-02'
select * from wiki_mkt_close_staging_inmem

set statistics time on
--select * from wiki_mkt_close_inmem
--update wiki_mkt_close_inmem set adj_close_ema = dbo.exp_moving_avg_scalar(null, adj_close, 21, 2)
--select * from wiki_mkt_close_inmem
--update wiki_mkt_close_inmem set adj_close_ema = NULL
--go
select * from wiki_mkt_close_inmem
go

--- This is what applying new data looks like

create proc dbo.wiki_mkt_insert_staging_data_ondisk
        @yesterday_asof_date date = '2016-06-01'
AS BEGIN		
DECLARE @new_asof_date DATE
set @new_asof_date = DATEADD(DAY, 1, @yesterday_asof_date)

delete from wiki_mkt_close_disk where asof_date = @new_asof_date

;with yesterday_close as (
          select ticker, asof_date, adj_close, adj_close_ema 
		  from wiki_mkt_close_disk 
		  where asof_date = @yesterday_asof_date
)
insert wiki_mkt_close_disk (ticker, asof_date, adj_close, adj_close_ema)
select coalesce(y.ticker, n.ticker) ticker 
     , @new_asof_date
	 , n.adj_close  
	 , dbo.exp_moving_avg_scalar (y.adj_close_ema, n.adj_close, 21, 2.0)
	from yesterday_close y
	FULL outer join wiki_mkt_close_staging_inmem n  on y.ticker = n.ticker
END
go

delete from wiki_mkt_close_disk where asof_date = '2016-06-02'

exec dbo.wiki_mkt_insert_staging_data_ondisk         @yesterday_asof_date = '2016-06-01'

CREATE TYPE dbo.ticker_list  AS TABLE (ticker varchar(16) NOT NULL PRIMARY KEY NONCLUSTERED)
WITH (MEMORY_OPTIMIZED = ON)
GO

drop procedure if exists  dbo.wiki_mkt_insert_staging_data_inmem
go


CREATE PROCEDURE dbo.wiki_mkt_insert_staging_data_inmem 
	@yesterday_asof_date DATE NOT NULL 

WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC WITH
(
	TRANSACTION ISOLATION LEVEL = SNAPSHOT,
	LANGUAGE = N'English'
)

	DECLARE @new_asof_date DATE = DATEADD(DAY, 1, @yesterday_asof_date)

	DELETE FROM dbo.wiki_mkt_close_inmem where asof_date = @new_asof_date

	INSERT dbo.wiki_mkt_close_inmem (ticker, asof_date, adj_close, adj_close_ema)
	SELECT n.ticker ticker 
     , @new_asof_date
	 , n.adj_close  
	 , dbo.exp_moving_avg_scalar_nc (y.adj_close_ema, n.adj_close, 21, 2.0)
	FROM dbo.wiki_mkt_close_inmem y
	INNER JOIN dbo.wiki_mkt_close_staging_inmem n on y.ticker = n.ticker
	WHERE y.asof_date = @yesterday_asof_date

	-- any from the staging table that were not in yesterday's data 
	/*
	INSERT dbo.wiki_mkt_close_inmem (ticker, asof_date, adj_close, adj_close_ema)
	SELECT n.ticker ticker 
     , @new_asof_date
	 , n.adj_close  
	 , dbo.exp_moving_avg_scalar_nc (NULL, n.adj_close, 21, 2.0)
	FROM dbo.wiki_mkt_close_staging_inmem n 
	WHERE n.ticker NOT IN (SELECT ticker from dbo.wiki_mkt_close_inmem where asof_date = @new_asof_date)
	 */

	declare @tickers_from_yesterday dbo.ticker_list;
	insert @tickers_from_yesterday 
	SELECT ticker from dbo.wiki_mkt_close_inmem where asof_date = @yesterday_asof_date

	INSERT dbo.wiki_mkt_close_inmem (ticker, asof_date, adj_close, adj_close_ema)
	SELECT n.ticker ticker 
     , @new_asof_date
	 , n.adj_close  
	 , dbo.exp_moving_avg_scalar_nc (NULL, n.adj_close, 21, 2.0)
	FROM dbo.wiki_mkt_close_staging_inmem n 
	WHERE n.ticker NOT IN (SELECT ticker FROM @tickers_from_yesterday)

	declare @tickers_from_today dbo.ticker_list;
	insert @tickers_from_today
	SELECT ticker from dbo.wiki_mkt_close_inmem where asof_date = @new_asof_date

	INSERT dbo.wiki_mkt_close_inmem (ticker, asof_date, adj_close, adj_close_ema)
	SELECT n.ticker ticker 
     , @new_asof_date
	 , NULL -- NULL because there's no data today
	 , null -- NULL because there's no data today
	FROM dbo.wiki_mkt_close_inmem n 
	WHERE n.ticker NOT IN (SELECT ticker from @tickers_from_today)
	
	RETURN 0;
END;
GO
set nocount off

DELETE from dbo.wiki_mkt_close_inmem where asof_date ='2016-06-02'

exec dbo.wiki_mkt_insert_staging_data_inmem         @yesterday_asof_date = '2016-06-01'

select * from wiki_mkt_close_inmem where asof_date ='2016-06-02'
